import pathUtils from 'node:path';
import debug from 'debug';
import typescript from 'typescript';
import { PROJECT_NAME } from "./constants.js";
/* eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-call*/
const log = debug(PROJECT_NAME);
const isBaseDir = (baseDir, testDir) => {
    const relative = pathUtils.relative(baseDir, testDir);
    return relative ? !relative.startsWith('..') && !pathUtils.isAbsolute(relative) : true;
};
export class Resolver {
    constructor(ts, compilerOptions, alias) {
        this.ts = ts;
        this.compilerOptions = compilerOptions;
        // Build matchers from compiler options
        this.matchers = [
            (moduleName, sourceFileName) => {
                const retVal = this.resolveModule(sourceFileName, moduleName);
                if (retVal) {
                    log(`${sourceFileName}: Rewrite module ${moduleName} to ${retVal}`);
                    return retVal;
                }
                return null;
            },
        ];
        if (alias) {
            // Build matchers from alias property of the plugin
            this.matchers.push(...Object.entries(alias).map(([from, to]) => {
                const fromRegex = new RegExp(from);
                return function (moduleName, sourceFileName) {
                    if (fromRegex.test(moduleName)) {
                        const retVal = moduleName.replace(fromRegex, to);
                        log(`${sourceFileName}: Rewrite module ${moduleName} to ${retVal} with regex /${from}/ => ${to}`);
                        return retVal;
                    }
                    return null;
                };
            }));
        }
        this.matchers.push();
    }
    resolve(moduleName, sourceFileName) {
        let retVal = null;
        this.matchers.forEach(function (matcher) {
            const result = matcher(moduleName, sourceFileName);
            if (result != null) {
                retVal = result;
            }
        });
        return retVal;
    }
    resolveModule(sourceFilePath, moduleName) {
        const { resolvedModule } = this.ts.resolveModuleName(moduleName, sourceFilePath, this.compilerOptions, this.ts.sys);
        if (!resolvedModule || resolvedModule.packageId) {
            return null;
        }
        const resolvedFile = pathUtils.parse(resolvedModule.resolvedFileName);
        const resolvedFileBaseWithoutExt = resolvedFile.name;
        const resolvedFileExt = resolvedFile.ext;
        let resolvedFileDir = resolvedFile.dir;
        let sourceFileDir = pathUtils.dirname(sourceFilePath);
        if (this.compilerOptions.rootDirs) {
            let fileRootDir;
            let moduleRootDir;
            this.compilerOptions.rootDirs.forEach(function (rootDir) {
                if (isBaseDir(rootDir, resolvedFileDir) && (!moduleRootDir || rootDir.length > moduleRootDir.length)) {
                    moduleRootDir = rootDir;
                }
                if (isBaseDir(rootDir, sourceFileDir) && (!fileRootDir || rootDir.length > fileRootDir.length)) {
                    fileRootDir = rootDir;
                }
            });
            /* Remove base dirs to make relative to root */
            if (fileRootDir && moduleRootDir) {
                sourceFileDir = pathUtils.relative(fileRootDir, sourceFileDir);
                resolvedFileDir = pathUtils.relative(moduleRootDir, resolvedFileDir);
            }
        }
        const normalisedResolvedFileDir = pathUtils.relative(sourceFileDir, resolvedFileDir);
        let normalisedResolvedFileBase = resolvedFileBaseWithoutExt;
        if (this.isESM()) {
            if (resolvedFileExt === '.ts') {
                normalisedResolvedFileBase += '.js';
            }
            else if (resolvedFileExt === '.mts') {
                normalisedResolvedFileBase += '.mjs';
            }
            else {
                normalisedResolvedFileBase += resolvedFileExt;
            }
        }
        let normalisedResolvedFilePath = pathUtils.join(normalisedResolvedFileDir, normalisedResolvedFileBase);
        if (!normalisedResolvedFilePath.startsWith('.')) {
            normalisedResolvedFilePath = `./${normalisedResolvedFilePath}`;
        }
        return normalisedResolvedFilePath;
    }
    isESM() {
        return (this.compilerOptions.module !== typescript.ModuleKind.None &&
            this.compilerOptions.module !== typescript.ModuleKind.CommonJS &&
            this.compilerOptions.module !== typescript.ModuleKind.AMD &&
            this.compilerOptions.module !== typescript.ModuleKind.UMD &&
            this.compilerOptions.module !== typescript.ModuleKind.System);
    }
}
//# sourceMappingURL=resolver.js.map